
#include "Rights.h"
#include "CommonFunctions.h"
#include "Strings.h"
#include "Ordering.h"
#include "Storage.h"
#include "NameTable.h"
#include "MMS_Filter.h"
#include "MMS.h"


/////////////////////////////////////////////////////////////////////
CUserRightsProp::TTypeTable CUserRightsProp::TypeTable[] = {
	{0x00, "0x00"},
	{0x01, ""},
	{0x02, ""},
	{0x03, " "},
	{0x04, ""},
	{0x05, " "},
	{0x06, ""},
	{0x07, "  "},
	{0x08, " "},
	{0x09, "   "},
	{0x0a, "  "},
	{0x0b, " "},
	{0x0c, "  "},
	{0x0d, "  \"\""},
	{0x0e, ""},
	{0x0f, "  "},
	{0x10, ""},
	{0x11, "   "},
	{0x12, "   OLE Automation "},
	{0x13, ""},
	{0x14, "  "},
	{0x15, "0x15"},
	{0x16, "0x16"},
	{0x17, "0x17"},
	{0x18, "0x18"},
	{0x19, "  "},
	{0x1a, "  "},
	{0x1b, "     "},
	{0x1c, "   "},
	{0x1d, "      "},
	{0x1e, "   "},
	{0x1f, "     "},
	{0x20, "    "},
	{0x21, "   "},
	{0x22, "  "},
	{0x23, "  "},
	{0x24, " "},
	{0x25, "  "},
	{0x26, "   "},
	{0x27, "  "},
	{0x28, "  "},
	{0x29, "  "},
	{0x2a, "/ "},
	{0x2b, " "},
	{0x2c, "   "},
	{0x2d, " "},
	{0x2e, " Web "},
	{0x00, NULL}
};

CUserRightsProp::CUserRightsProp()
{
	Type = 0;
	Value = 0;
}

const char* CUserRightsProp::TypeAsString()
{
	TTypeTable* ptr = TypeTable;
	while( ptr->sID != NULL )
	{
		if( ptr->ID == Type )
			return ptr->sID;
		ptr++;
	}
	
	Msg(0, "ERR:    . ID = %i", Type);

	char str[32];
	ltoa(Type, str, 16);
	return str;
}

const char* CUserRightsProp::ValueAsString()
{
	if( Value == 0 )
		return "";
	else
		return "";
}

void CUserRightsProp::BeautyString(CString& String, int level)
{
	add_parameter(String, level, TypeAsString(), ValueAsString(), "", true);
}

void CUserRightsProp::Read(CStorage& Storage)
{
	IStream* pStream = Storage.pStream;

	pStream->Read(&Type, sizeof(Type), NULL);
	pStream->Read(&Value, sizeof(Value), NULL);
}

////////////////////////////////////////////////////////////////////
CUserRightsBlock::TBlockType CUserRightsBlock::BlockTypes[] = {
	{0x01, ""},
	{0x03, ""},
	{0x06, ""},
	{0x09, ""},
	{0x0b, " "},
	{0x10, ""},
	{0x11, ""},
	{0x12, ""},
	{0x17, " "},
	{0x18, ""},
	{0x19, " "},
	{0x1a, ""},
	{0x1b, ""},
	{0x1c, " "},
	{0x1f, "0x1F"},
	{0x00, NULL}
};

CUserRightsBlock::CUserRightsBlock()
{
	Type = 0;
	MetadataID = 0;
	nProps = 0;
}

CUserRightsBlock& CUserRightsBlock::operator=(CUserRightsBlock& Block)
{
	Type = Block.Type;
	MetadataID = Block.MetadataID;
	nProps = Block.nProps;
	return *this;
}

const char* CUserRightsBlock::TypeAsString()
{
	TBlockType* ptr = BlockTypes;
	while( ptr->sID != NULL )
	{
		if( ptr->ID == Type ) return ptr->sID;
		ptr++;
	}

	Msg(0, "ERR:   . ID = %i", Type);

	char str[32];
	ltoa(Type, str, 16);
	return str;
}

void CUserRightsBlock::BeautyString(CString& String, int level)
{
	CString& ObjectName = MainMetadataStream->ObjectIDtoName(MetadataID);

	add_parameter(String, level, TypeAsString(), ObjectName, NULL, true);
	add_offset(String, level);
	String += "{\r\n";

	for( int i = 0; i < nProps; i++ )
	{
		Props[i].BeautyString(String, level+1);
	}

	add_offset(String, level);
	String += "}\r\n";
}

void CUserRightsBlock::Read(CStorage& Storage)
{
	IStream* pStream = Storage.pStream;
	
	pStream->Read(&Type, sizeof(Type), NULL);
	pStream->Read(&MetadataID, sizeof(MetadataID), NULL);
	pStream->Read(&nProps, sizeof(nProps), NULL);

	Props.SetSize(nProps);
	for( int i = 0; i < nProps; i++ )
	{
		Props[i].Read(Storage);
	}
}


////////////////////////////////////////////////////////////////////
CUserRights::CUserRights()
{
	ID = 0x06;
	nBlocks = 0;
}

void CUserRights::BeautyString(CString& String)
{
	add_parameter(String, 0, "", Description, "");

	for( int i = 0; i < nBlocks; i++ )
	{
		Blocks[i].BeautyString(String, 0);
	}
}

void CUserRights::Read(CStorage& Storage)
{
	IStream* pStream = Storage.pStream;
	
	pStream->Read(&ID, sizeof(ID), NULL);
	ReadSizedString(Description, pStream);
	
	pStream->Read(&nBlocks, sizeof(nBlocks), NULL);

	Msg(2, "   %i ", nBlocks);
	Blocks.SetSize(nBlocks);
	for( int i = 0; i < nBlocks; i++ )
	{
		Blocks[i].Read(Storage);
	}
}

void CUserRights::WriteToFile(CStorage& Storage, CString FileName)
{
	CString String;
	BeautyString(String);
	Storage.CopyStringToFile(String, FileName);
}


void DecompileUserRights(CString& Dir, CStorage& Storage)
{
	CNameTableRecord* nt_rec = NameTable.Find(NT_UserRights);
	CString DestDir = nt_rec->PrepareDirName(Dir);

	MMS_Filter.Down();
	if( MMS_Filter.Test(NULL, nt_rec) )
	{
		CStorage RtStorage(Storage);

		if( RtStorage.Open(nt_rec->PrepareStorageName()) )
		{
			CMMSNodeList* CC;
			CC = (CMMSNodeList*)RtStorage.ParseStream(CString("Container.Contents"), false);
			if( CC != NULL )
			{
				CObjectOrdering Ordering(1);

				CreateDirectory(DestDir);

				POSITION pos = CC->GetHeadPosition();
				while( pos != NULL )
				{
					CMMSObject* cc_obj = CC->GetNext(pos);
					CUserRights Rights;
					CString& StreamName = cc_obj->GetPropByNum(0)->sID;
					CString& RightsName = cc_obj->GetPropByNum(1)->sID;
					CString FileName = nt_rec->PrepareFileName(Dir, cc_obj->GetPropByNum(1));

					Msg(2, "  '%s'", RightsName);
					RtStorage.OpenStream(StreamName);
					try
					{
						Rights.Read(RtStorage);
					}
					catch(...)
					{
						Msg(0, "ERR:      %s", RightsName);
					}

					Rights.WriteToFile(RtStorage, FileName);
					Ordering.Add(RightsName, StreamName);
				}
				delete CC;
				Ordering.WriteFile(RtStorage, DestDir);
			}
		}
	}
	MMS_Filter.Up();
}